﻿using System;
using System.Collections.Generic;
using System.IdentityModel.Services;
using System.IdentityModel.Tokens;
using System.Linq;
using System.Security.Claims;
using System.Web;
using System.Web.Mvc;
using Thinktecture.IdentityModel.Oidc;
using Thinktecture.IdentityModel.Web;

namespace OIDCClient.Controllers
{
    public class HomeController : Controller
    {
        //
        // GET: /Home/

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult OidcError(string error)
        {
            ViewBag.Message = error;

            return View();
        }

        public ActionResult Logout()
        {
            FederatedAuthentication.SessionAuthenticationModule.SignOut();
            return Redirect("~/");
        }

        public ActionResult SLO()
        {
            var config = OidcClientConfigurationSection.Instance;

            var idTokenCookie = new ProtectedCookie(ProtectionMode.MachineKey);
            var idToken = idTokenCookie.Read("oidcIDToken");
            ProtectedCookie.Delete("oidcIDToken");
            ProtectedCookie.Delete("oidcSessionState");
            string logoutUrl = config.Endpoints.Logout + "?id_token_hint=" + idToken;
            FederatedAuthentication.SessionAuthenticationModule.SignOut();
            return Redirect(logoutUrl);
        }

        [Authorize]
        public ActionResult Claims()
        {
            var config = OidcClientConfigurationSection.Instance;

            ViewBag.Message = "Claims:";
            var idTokenCookie = new ProtectedCookie(ProtectionMode.MachineKey);
            var idToken = idTokenCookie.Read("oidcIDToken");

            var opUrl = config.Endpoints.SessionLogout + "?id_token_hint=" + idToken;
            ViewData["opUrl"] = opUrl;
            ViewData["logoutUrl"] = Request.Url.Scheme + "://" + Request.Url.Host + "/Logout";
            return View(ClaimsPrincipal.Current);
        }

        public ActionResult RPIFrame()
        {
            var config = OidcClientConfigurationSection.Instance;

            ViewData["clientId"] = config.ClientId;

            var accessTokenCookie = new ProtectedCookie(ProtectionMode.MachineKey);
            var oidcSessionStateStr = accessTokenCookie.Read("oidcSessionState");
            var session_state = oidcSessionStateStr; //idToken.Claims.FirstOrDefault(claim => claim.Type == "session_state"); ;

            //var sessionStateCookie = new ProtectedCookie(ProtectionMode.MachineKey);
            //var session_state = sessionStateCookie.Read("op_session_state");

            ViewData["session_state"] = session_state;

            //ViewData["origin"] = Request.Url.Scheme + "://" + Request.Url.Host;

            ViewData["OPAddress"] = config.Endpoints.TargetOP;
            ViewData["targetOrigin"] = config.Endpoints.SessionLogout;


            return View();
        }

    }
}
